import {
  Domain,
  Identifier,
  Type,
  Value,
} from '@mobicoop/configuration-module';
import { NotFoundException } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { AuthConfig } from '@src/config/auth.config';
import { CarpoolConfig } from '@src/config/carpool.config';
import { Config } from '@src/config/config';
import { GeographyConfig } from '@src/config/geography.config';
import { MatchConfig } from '@src/config/match.config';
import { PaginationConfig } from '@src/config/pagination.config';

@Injectable()
export class ConfigurationsManagerService {
  constructor(private readonly configService: ConfigService) {}

  list = (): Config[] => {
    return [
      {
        ...(this.configService.get<AuthConfig>('auth') as AuthConfig),
        domain: Domain.AUTH,
      },
      {
        ...(this.configService.get<CarpoolConfig>('carpool') as CarpoolConfig),
        domain: Domain.CARPOOL,
      },
      {
        ...(this.configService.get<GeographyConfig>(
          'geography',
        ) as GeographyConfig),
        domain: Domain.GEOGRAPHY,
      },
      {
        ...(this.configService.get<MatchConfig>('match') as MatchConfig),
        domain: Domain.MATCH,
      },
      {
        ...(this.configService.get<PaginationConfig>(
          'pagination',
        ) as PaginationConfig),
        domain: Domain.PAGINATION,
      },
    ];
  };

  identifierType = (identifier: Identifier): Type => {
    const configs: Config[] = this.list();
    const configuration: Config | undefined = configs.find(
      (config: Config) =>
        config.domain === identifier.domain &&
        this._hasProperty(identifier.key, config),
    );
    if (!configuration)
      throw new NotFoundException('Configuration item not found');
    return this.configurationType(
      this._getValue(identifier.key, configuration),
    );
  };

  configurationType = (value: any): Type => {
    if (Array.isArray(value)) return this._configurationTypeArray(value);
    switch (typeof value) {
      case 'number':
        if (this._isInt(value)) return Type.INT;
        return Type.FLOAT;
      case 'boolean':
        return Type.BOOLEAN;
      default:
        if (value.indexOf(',') === -1) return Type.STRING;
        return this._configurationTypeArray(value.split(','));
    }
  };

  _configurationTypeArray = (value: Array<string | number>): Type => {
    return value.every((item) => typeof item === 'number' && this._isInt(item))
      ? Type.INT_ARRAY
      : value.every((item) => typeof item === 'number')
        ? Type.FLOAT_ARRAY
        : Type.STRING_ARRAY;
  };

  cast = (value: string, type: Type): Value => {
    switch (type) {
      case Type.BOOLEAN:
        return value === 'true';
      case Type.INT:
        return parseInt(value);
      case Type.FLOAT:
        return parseFloat(value);
      case Type.INT_ARRAY:
        return value.split(',').map((item: string) => parseInt(item));
      case Type.FLOAT_ARRAY:
        return value.split(',').map((item: string) => parseFloat(item));
      case Type.STRING_ARRAY:
        return value.split(',');
      default:
        return value;
    }
  };

  private _hasProperty = <T>(property: string, configuration: T): boolean => {
    let key: keyof typeof configuration;
    for (key in configuration) {
      if (key !== 'domain' && key === property) return true;
    }
    return false;
  };

  private _getValue = <T>(property: string, configuration: T): any => {
    let key: keyof typeof configuration;
    for (key in configuration) {
      if (key !== 'domain' && key === property) return configuration[key];
    }
  };

  private _isInt = (number: number): boolean => number % 1 === 0;
}
