import { Domain, Identifier, Type } from '@mobicoop/configuration-module';
import { NotFoundException } from '@mobicoop/ddd-library';
import { ConfigurationsManagerService } from '@modules/configuration/core/application/services/configurations-manager.service';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';
import { Config } from '@src/config/config';

const mockConfigService = {
  get: jest.fn().mockImplementation((domain: string) => {
    switch (domain) {
      case 'auth':
        return {
          encryptionAlgorithm: 'BCRYPT',
        };
      case 'carpool':
        return {
          departureTimeMargin: 900,
          role: 'passenger',
          seatsProposed: 3,
          seatsRequested: 1,
          strictFrequency: false,
        };
      case 'geography':
        return {
          georouterType: 'graphhopper',
          georouterUrl: 'http://localhost:8989',
          geocoderLang: 'fr',
          geocoderMinConfidence: 0.3,
          geocoderMaxResultsPerProvider: 5,
          geocoderMaxResultsPerType: 5,
          geocoderSanitize: true,
          geocoderConsolidate: true,
          geocoderProximity: 5,
          geocoderPopulationPrioritizerCoef: 100,
          geocoderProviders: ['provider1', 'provider2'],
          geocoderProvidersFallback: ['provider3'],
        };
      case 'match':
        return {
          algorithm: 'PASSENGER_ORIENTED',
          remoteness: 15000,
          useProportion: true,
          proportion: 0.3,
          useAzimuth: true,
          azimuthMargin: 10,
          maxDetourDistanceRatio: 0.3,
          maxDetourDurationRatio: 0.3,
        };
      case 'pagination':
        return {
          perPage: 10,
        };
    }
  }),
};

describe('Configurations Manager Service', () => {
  let configurationsManagerService: ConfigurationsManagerService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        ConfigurationsManagerService,
      ],
    }).compile();

    configurationsManagerService = module.get<ConfigurationsManagerService>(
      ConfigurationsManagerService,
    );
  });

  it('should be defined', () => {
    expect(configurationsManagerService).toBeDefined();
  });

  it('should return the list of configuration elements', () => {
    const list: Config[] = configurationsManagerService.list();
    expect(list).toHaveLength(5);
  });

  describe('identifierType', () => {
    it('should return the type of a configuration item for a given identifier', () => {
      const identifier: Identifier = {
        domain: Domain.CARPOOL,
        key: 'seatsProposed',
      };
      const type: Type =
        configurationsManagerService.identifierType(identifier);
      expect(type).toBe(Type.INT);
    });
    it('should throw if configuration item is not found', () => {
      const dentifier: Identifier = {
        domain: Domain.MATCH,
        key: 'maxDetour',
      };
      expect(() => {
        configurationsManagerService.identifierType(dentifier);
      }).toThrow(NotFoundException);
    });
  });

  describe('Type', () => {
    it('should return the configuration type of an int', () => {
      expect(configurationsManagerService.configurationType(3)).toBe(Type.INT);
    });
    it('should return the configuration type of a float', () => {
      expect(configurationsManagerService.configurationType(3.5)).toBe(
        Type.FLOAT,
      );
    });
    it('should return the configuration type of a boolean', () => {
      expect(configurationsManagerService.configurationType(true)).toBe(
        Type.BOOLEAN,
      );
    });
    it('should return the configuration type of a string', () => {
      expect(configurationsManagerService.configurationType('role')).toBe(
        Type.STRING,
      );
    });
    it('should return the configuration type of a string array', () => {
      expect(
        configurationsManagerService.configurationType(['test', 'test2']),
      ).toBe(Type.STRING_ARRAY);
      expect(configurationsManagerService.configurationType('test,test2')).toBe(
        Type.STRING_ARRAY,
      );
    });
    it('should return the configuration type of an int array', () => {
      expect(configurationsManagerService.configurationType([2, 3])).toBe(
        Type.INT_ARRAY,
      );
    });
    it('should return the configuration type of a float array', () => {
      expect(configurationsManagerService.configurationType([1.2, 3.6])).toBe(
        Type.FLOAT_ARRAY,
      );
    });
  });

  describe('cast', () => {
    it('should cast a string to int', () => {
      expect(configurationsManagerService.cast('1', Type.INT)).toBe(1);
    });
    it('should cast a string to float', () => {
      expect(configurationsManagerService.cast('1.5', Type.FLOAT)).toBe(1.5);
    });
    it('should cast a string to boolean', () => {
      expect(
        configurationsManagerService.cast('true', Type.BOOLEAN),
      ).toBeTruthy();
    });
    it('should not cast a string and return it as is', () => {
      expect(configurationsManagerService.cast('role', Type.STRING)).toBe(
        'role',
      );
    });
    it('should cast a string to an array of strings', () => {
      expect(
        configurationsManagerService.cast('test,test2', Type.STRING_ARRAY),
      ).toStrictEqual(['test', 'test2']);
    });
    it('should cast a string to an array of ints', () => {
      expect(
        configurationsManagerService.cast('1,2', Type.INT_ARRAY),
      ).toStrictEqual([1, 2]);
    });
    it('should cast a string to an array of floats', () => {
      expect(
        configurationsManagerService.cast('1.2,2.3', Type.FLOAT_ARRAY),
      ).toStrictEqual([1.2, 2.3]);
    });
  });
});
