import { Domain } from '@mobicoop/configuration-module';
import { NotFoundException, RpcExceptionCode } from '@mobicoop/ddd-library';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';
import { GetConfigurationGrpcController } from '@modules/configuration/interface/grpc-controllers/get-configuration.grpc.controller';
import { QueryBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const mockQueryBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => '3')
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockConfigurationMapper = {
  toResponse: jest.fn().mockImplementationOnce(() => ({
    domain: Domain.CARPOOL,
    key: 'seatsProposed',
    value: '3',
  })),
};

describe('Get Configuration Grpc Controller', () => {
  let getConfigurationGrpcController: GetConfigurationGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: QueryBus,
          useValue: mockQueryBus,
        },
        {
          provide: ConfigurationMapper,
          useValue: mockConfigurationMapper,
        },
        GetConfigurationGrpcController,
      ],
    }).compile();

    getConfigurationGrpcController = module.get<GetConfigurationGrpcController>(
      GetConfigurationGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(getConfigurationGrpcController).toBeDefined();
  });

  it('should return a configuration item', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockConfigurationMapper, 'toResponse');
    const response = await getConfigurationGrpcController.get({
      domain: Domain.CARPOOL,
      key: 'seatsProposed',
    });
    expect(response.value).toBe('3');
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockConfigurationMapper.toResponse).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if configuration is not found', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockConfigurationMapper, 'toResponse');
    expect.assertions(4);
    try {
      await getConfigurationGrpcController.get({
        domain: Domain.CARPOOL,
        key: 'price',
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.NOT_FOUND);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockConfigurationMapper.toResponse).toHaveBeenCalledTimes(0);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockConfigurationMapper, 'toResponse');
    expect.assertions(4);
    try {
      await getConfigurationGrpcController.get({
        domain: Domain.CARPOOL,
        key: 'someValue',
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockConfigurationMapper.toResponse).toHaveBeenCalledTimes(0);
  });
});
